/***************************************************************************
                          kalbum.cpp  -  description
                             -------------------
    begin                : Tue Jan 22 14:13:58 CET 2002
    copyright            : (C) 2002 by Graham Morrison
    email                : kalbum@paldandy.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <qlist.h>
#include <qprogressbar.h>
#include <kstatusbar.h>

// application specific includes
#include "kalbum.h"
#include "kalbumview.h"
#include "kalbumdoc.h"
#include "kalbumitem.h"

#define ID_STATUS_MSG 1

KalbumApp::KalbumApp(QWidget* , const char* name):KMainWindow(0, name)
{

  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(true);
  filePrint->setEnabled(false);
	fileOpenRecent->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
	editMoveUp->setEnabled(false);
	editMoveDown->setEnabled(false);	
  editPhoto->setEnabled(false);
//  editPaste->setEnabled(false);
}

KalbumApp::~KalbumApp()
{

}

void KalbumApp::initActions()
{
  fileNewWindow = new KAction(i18n("New &Window"), "window_new", 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
	fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
//  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

//My Actions------------------

	fileOpenPhoto = new KAction(i18n("Insert &Photo"), "addphoto", 0, this, SLOT(slotFileOpenPhoto()),
			actionCollection(), "file_open_photo");
	fileDocInfo = new KAction(i18n("Album &Information"), "info", 0, this, SLOT(slotFileDocInfo()),
			actionCollection(), "file_doc_info");
	fileExportHTML = new KAction(i18n("&Web Page Album"), "exphtml", 0, this, SLOT(slotFileExportHTML()),
			actionCollection(), "file_export_html");
	editMoveUp = new KAction(i18n("Moves Photos &Right"), "forward", 0, this, SLOT(slotEditMoveUp()),
			actionCollection(), "edit_move_up");
	editMoveDown = new KAction(i18n("Moves Photos &Left"), "back", 0, this, SLOT(slotEditMoveDown()),
			actionCollection(), "edit_move_down");
  editPhoto = new KAction(i18n("Edit Photo &Properties"), "edit", 0, this, SLOT(slotEditPhoto()),
      actionCollection(), "edit_photo");

//  insertAlbum = new KAction(i18n("Insert &Album"), "thumbnail", 0, this, SLOT(slotInsertAlbum()),
//			actionCollection(), "insert_album");

//-----------------------------	

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
//  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  editPhoto->setStatusText(i18n("Edits the properties for the selected photo"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

	fileOpenPhoto->setStatusText(i18n("Inserts a photo into the current album"));
	fileDocInfo->setStatusText(i18n("Sets general information for the album"));
	editMoveUp->setStatusText(i18n("Moves selected photo(s) right"));
	editMoveDown->setStatusText(i18n("Moves selected photo(s) left"));
//	insertAlbum->setStatusText(i18n("Inserts another album into the current one"));

  // use the absolute path to your kalbumui.rc file for testing purpose in createGUI();
  createGUI();

}


void KalbumApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KalbumApp::initDocument()
{
  doc = new KalbumDoc(this);
  doc->newDocument();
}

void KalbumApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KalbumView(this);
  doc->addView(view);
  setCentralWidget(view);	
  setCaption(doc->URL().fileName(),false);

}

void KalbumApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


KalbumDoc *KalbumApp::getDocument() const
{
  return doc;
}

void KalbumApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
//  fileOpenRecent->saveEntries(config,"Recent Files");
}


void KalbumApp::readOptions()
{
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();



  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
	fileOpenRecent->setMaxItems(5);
  fileOpenRecent->clearURLList();
	
//	fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void KalbumApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void KalbumApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}		

bool KalbumApp::queryClose()
{
  return doc->saveModified();
}

bool KalbumApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KalbumApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KalbumApp *new_window= new KalbumApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    doc->newDocument();		
    setCaption(doc->URL().fileName(), false);
		fileSave->setEnabled(false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.kalbum |Albums\n*|All files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
			doc->newDocument();	
      doc->openDocument(url);
//			view->loadPhotoAlbum(url);
      setCaption(url.fileName(), false);
      fileOpenRecent->addURL( url );
    }
  }
	fileSave->setEnabled(true);
  slotStatusMsg(i18n("Ready."));
}


void KalbumApp::slotInsertAlbum()
{
	slotStatusMsg(i18n("Inserting album..."));
	
	KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.kalbum |Albums\n*|All files"), this, i18n("Open File..."));
  if(!url.isEmpty())
  {
  	doc->openDocument(url);
//	view->loadPhotoAlbum(url);
    fileOpenRecent->addURL( url );
  }
  doc->setModified(true);
  slotStatusMsg(i18n("Ready."));	
}

void KalbumApp::slotFileOpenPhoto()
{
  KURL::List::ConstIterator it;
	slotStatusMsg(i18n("Opening photo..."));

	KURL::List url=KFileDialog::getOpenURLs(QString::null,
        i18n("*.png *.jpg *.bmp |Pictures\n*|All files"), this, i18n("Open Photo..."));

  for (it = url.begin(); it != url.end(); ++it){
		if((*it).isValid()){		

			doc->openPhoto(*it);
//    	view->addPhotoItem(*it);
//    	setCaption(*it.fileName(), false);
   	}
	}
  slotStatusMsg(i18n("Ready."));
}



void KalbumApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
//
//    cout << url.fileName();
//		doc->newDocument();
//		doc->openDocument(url);
    fileSave->setEnabled(true);
//		view->loadPhotoAlbum(url);
    setCaption(url.fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
  doc->saveDocument(doc->URL());

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileSaveAs()
{
	QString checkFileEnd;
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));


	if(!url.isEmpty())
  {

//   	view->savePhotoAlbum(url);

    doc->saveDocument(url);

    checkFileEnd = url.fileName();
		checkFileEnd = checkFileEnd.right(7);

		if ( checkFileEnd != FILEEX )
		    url.setFileName(url.fileName() + FILEEX);

   fileOpenRecent->addURL(url);
    setCaption(url.fileName(),doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileExportHTML()
{
  slotStatusMsg(i18n("Exporting Album as HTML..."));
/*
 KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Export as..."));


  if(!url.isEmpty())
 {
 		doc->exportHTML(url);
 }
*/
  doc->exportHTML();

  slotStatusMsg(i18n("Ready."));

}




void KalbumApp::slotFileDocInfo()
{

	view->setDocInfo();
}


void KalbumApp::slotEditMoveUp()
{

	view->editMoveUp();

}

void KalbumApp::slotEditMoveDown()
{

	view->editMoveDown();
}

void KalbumApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
}

void KalbumApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

	view->cutPhotoItem();

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

	view->copyPhotoItem();

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

	view->pastePhotoItem();

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotEditPhoto()
{
  slotStatusMsg(i18n("Editing Photo Properties..."));

  view->editPhotoItem();

  slotStatusMsg(i18n("Ready."));
}


void KalbumApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KalbumApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void KalbumApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

