/***************************************************************************
                          kalbumview.cpp  -  description
                             -------------------
    begin                : Tue Jan 22 14:13:58 CET 2002
    copyright            : (C) 2002 by Graham Morrison
    email                : kalbum@paldandy.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>

// Kalbum
#include <stdio.h>
#include <kmessagebox.h>
#include <kiconview.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <qlist.h>
#include <kpixmap.h>
#include <qmultilineedit.h>
#include <qlineedit.h>
#include <qdatetime.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kiconeffect.h>
#include <klocale.h>
#include <qiconview.h>
#include <qclipboard.h>
#include <qdatastream.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <ksimpleconfig.h>
#include <qbuffer.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <kiconloader.h>
#include <qradiobutton.h>
#include <klineedit.h>
#include <qdir.h>
#include <kimageeffect.h>
#include <qslider.h>
#include <kurlrequester.h>
#include <qradiobutton.h>
#include <kpixmap.h>
#include <kstddirs.h>


// application specific includes
#include "kalbumview.h"
#include "kalbumdoc.h"
#include "kalbum.h"
#include "kalbumitem.h"
#include "photoalbum.h"
#include "kalmimesource.h"
#include "exportweb.h"

/* XPM */
static char * red_xpm[] = {
"1 1 2 1",
" 	c None",
".	c #FF0000",
"."};

#define MAX(A,B) (((A)>(B)) ? (A) : (B))
#define ITEMPIXX	350
#define ITEMPIXY	350
#define PHOTOSZX	450
#define PHOTOSZY  320
#define THUMBSZX	125
#define THUMBSZY	110
#define THUMBFMT	"PNG"
#define PHOTOFMT	"JPEG"
#define MAXPHOTO	4 // Number of zeros for inserting photo numbers into HTML

#define THUMBHTM	"thm_"
#define PHOTOHTM	"pho_"

#define KWDPHCNT	"<!-- {Kalbum Photos="
#define KWDTHCNT	"<!-- {Kalbum Thumbs="
#define KWDVERSI	"<!-- {Kalbum Vers="
#define KWDTHUMB	"<!-- {Kalbum Thumbnail="
#define KWDTITLE  "{Kalbum Title="

// Pop Up Menu Number defaults
#define POPUPCUT  1
#define POPUPCPY  2
#define POPUPPST  3
#define POPUPLFT  4
#define POPUPRGT  5
#define POPUPEDT  6

QList<PhotoAlbum> *KalbumView::album = 0L;
KalbumCredits KalbumView::credits;


KalbumView::KalbumView(QWidget *parent, const char *name) :KalbumViewDlg(parent, name)
{

    setBackgroundMode(PaletteBase);
    setAcceptDrops(true);

    if (!album)
    {
        album = new QList<PhotoAlbum>;
    }

    album->setAutoDelete(true);

    popIconView = new KPopupMenu();
    popIconView->insertItem(SmallIconSet("editcut"),i18n("Cut"),this,SLOT(slotCut()), 1, POPUPCUT);
    popIconView->insertItem(SmallIconSet("editcopy"),i18n("Copy"),this,SLOT(slotCopy()), 2, POPUPCPY);
    popIconView->insertItem(SmallIconSet("editpaste"),i18n("Paste"),this,SLOT(slotPaste()), 3, POPUPPST);
    popIconView->insertSeparator(4);
    popIconView->insertItem(SmallIconSet("back"),i18n("Move Left"),this,SLOT(slotMoveLeft()), 5, POPUPLFT);
    popIconView->insertItem(SmallIconSet("forward"),i18n("Move Right"),this,SLOT(slotMoveRight()), 6, POPUPRGT);
    popIconView->insertSeparator(7);
    popIconView->insertItem(SmallIconSet("edit"),i18n("Edit Photo"),this,SLOT(slotEditPhoto()), 8, POPUPEDT);

    popIconView->setItemEnabled(POPUPCPY, false);
    popIconView->setItemEnabled(POPUPCUT, false);

    popIconView->setItemEnabled(POPUPLFT, false);
    popIconView->setItemEnabled(POPUPRGT, false);
    popIconView->setItemEnabled(POPUPEDT, false);

}

KalbumView::~KalbumView()
{}


KalbumDoc *KalbumView::getDocument() const
{
    KalbumApp *theApp=(KalbumApp *) parentWidget();

    return theApp->getDocument();
}

void KalbumView::print(QPrinter *pPrinter)
{
    QPainter printpainter;
    printpainter.begin(pPrinter);

    // TODO: add your printing code here

    printpainter.end();
}

/** Returns Photo data for selected Icon */
PhotoAlbum *KalbumView::getPhoto(QIconViewItem *chosen)
{
    PhotoAlbum *photo;

    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (chosen == photo->iconitem)
        {
            return photo;
        }
    }

    return photo;


}

PhotoAlbum *KalbumView::nextPhoto(PhotoAlbum *photo)
{
    PhotoAlbum *tmp_photo;

    tmp_photo = getPhoto(photo->iconitem->nextItem());
    if (tmp_photo)
        return tmp_photo;
    else
        return getPhoto(photo->iconitem->iconView()->firstItem());

}

PhotoAlbum *KalbumView::prevPhoto(PhotoAlbum *photo)
{
    PhotoAlbum *tmp_photo;

    tmp_photo = getPhoto(photo->iconitem->prevItem());
    if (tmp_photo)
        return tmp_photo;
    else
        return getPhoto(photo->iconitem->iconView()->lastItem());

}


/** Adds a new photo into the album with an URL as an argument */
bool KalbumView::addPhotoItem(const KURL& url)
{
    PhotoAlbum *photo = new PhotoAlbum;
    photo->fileurl=url;

    photo=photo->dataPhoto();

    if (photo!=NULL)
    {
        photo->iconitem = new QIconViewItem( KIconView1, photo->title, photo->thumbnail);
        album->append(photo);
    }

    return true;

}

/** Overloaded for PhotoAlbum */
bool KalbumView::addPhotoItem(PhotoAlbum *photo)
{

    photo=photo->dataPhoto();

    if (photo!=NULL)
    {
        photo->iconitem = new QIconViewItem( KIconView1, photo->title, photo->thumbnail);
        album->append(photo);
    }

    return true;
}

/** Overloaded for PhotoAlbum - Inserts a photo */
bool KalbumView::addPhotoItem(PhotoAlbum *photo, PhotoAlbum *before)
{
    photo=photo->dataPhoto();

    if (photo!=NULL)
    {
        if (before == album->first())
        {
            photo->iconitem = new QIconViewItem( KIconView1, KIconView1->firstItem(), photo->title, photo->thumbnail);
            album->insert(0, photo);

            PhotoAlbum *tmpphoto = new PhotoAlbum;
            tmpphoto = getPhoto(KIconView1->firstItem());

            delete KIconView1->firstItem();
            tmpphoto->iconitem = new QIconViewItem( KIconView1, KIconView1->firstItem(), tmpphoto->title, tmpphoto->thumbnail);

        }
        else
        {
            int position;
            position = album->find(before);
            photo->iconitem = new QIconViewItem( KIconView1, before->iconitem->prevItem(), photo->title, photo->thumbnail);
            album->insert(position, photo);
        }


    }

    return true;

}

void KalbumView::deletePhotoItem(PhotoAlbum *photo)
{

    delete photo->iconitem;
    album->remove(photo);

}


void KalbumView::deleteAll()
{
    PhotoAlbum *photo = new PhotoAlbum;

    if (album!=NULL)
    {
        for (photo=album->first(); photo!=0; photo=album->next())
        {
            delete photo->iconitem;
            album->remove(photo);
            photo=album->first();
        }

        for (photo=album->first(); photo!=0; photo=album->next())
        {
            delete photo->iconitem;
            album->remove(photo);

        }
        KIconView1->arrangeItemsInGrid();
    }

}


void KalbumView::slotPhotoProp(QIconViewItem *chosen)
{
    KalbumItem *item = new KalbumItem;
    QPixmap *buffer = new QPixmap;
    PhotoAlbum *photo;
    QString fname;
    QString tmpfile;

    photo=getPhoto(chosen);


    // Uses the Icon as the Pixmap
    //  buffer=chosen->pixmap();

    // Uses the picture as the pixmap
    KIO::NetAccess::download( photo->fileurl, tmpfile);
    //  item->EditComment->setText(tmpfile);

    if (!(buffer->load(tmpfile))){
          buffer = photo->iconitem->pixmap();
    }

//    buffer->load(tmpfile);



    item->photo = photo;
    item->setCaption(photo->fileurl.fileName());
    item->EditComment->setText(photo->comment);
    item->LineEditTitle->setText(photo->title);
    item->photoDate->setDate(photo->date);
    item->RadioButton1->setChecked(photo->ignoredate);
    item->PixmapLabel2->setPixmap(*buffer);

    float s=MAX((float(photo->xsize)/ITEMPIXX), (float(photo->ysize)/ITEMPIXY));

    item->PixmapLabel2->setMaximumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );

    item->PixmapLabel2->setMinimumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );

    //KMessageBox::information(this, "Title", (QString)photo->ysize);



    //	item->TextLabelSize->setNum(photo->filesize/1024);


    if (item->exec())
    {
        photo=item->photo;
        photo->comment=item->EditComment->text();
        photo->title=item->LineEditTitle->text();
        photo->iconitem->setText(photo->title);
        photo->date=item->photoDate->getDate();
        photo->ignoredate = item->RadioButton1->isChecked();
    }

    KIconView1->arrangeItemsInGrid();
    delete item;

    KalbumDoc *doc;
    doc=getDocument();
    doc->setModified(true);

}

void KalbumView::setDocInfo()
{
    KalbumCreditsView *creditdlg = new KalbumCreditsView;

    if (!credits.albumName.isEmpty())
        creditdlg->albumName->setText(credits.albumName);
    if (credits.webAddress.isValid())
        creditdlg->webAddress->setText(credits.webAddress.url());
    if (!credits.authorName.isEmpty())
        creditdlg->authorName->setText(credits.authorName);
    if (!credits.authorEmail.isEmpty())
        creditdlg->authorEmail->setText(credits.authorEmail);
    if (credits.startDate.isValid())
        creditdlg->startDate->setDate(credits.startDate);
    if (credits.endDate.isValid())
        creditdlg->endDate->setDate(credits.endDate);
    if (!credits.albumDescription.isEmpty())
        creditdlg->albumDescription->setText(credits.albumDescription);

    creditdlg->RadioButton3->setChecked(credits.ignoredates);


    if (creditdlg->exec())
    {
        credits.albumName 				= creditdlg->albumName->text();
        credits.webAddress 				= creditdlg->webAddress->text();
        credits.authorName				=	creditdlg->authorName->text();
        credits.authorEmail				=	creditdlg->authorEmail->text();
        credits.startDate					= creditdlg->startDate->getDate();
        credits.endDate						= creditdlg->endDate->getDate();
        credits.albumDescription	=	creditdlg->albumDescription->text();
        credits.ignoredates       = creditdlg->RadioButton3->isChecked();
    }

}


QList<PhotoAlbum> KalbumView::getPhotoData()
{
    PhotoAlbum *photo = new PhotoAlbum;
    QList<PhotoAlbum> photolist;


    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {
            PhotoAlbum *phototmp = new PhotoAlbum;
            phototmp = photo->copy();
            photolist.append(phototmp);

        }

    }

    return photolist;


}


/** Copy the icon data to the Clipboard */
void KalbumView::copyPhotoItem()
{
    PhotoAlbum *photo = new PhotoAlbum;
    QList<PhotoAlbum> photolist;


    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {
            PhotoAlbum *phototmp = new PhotoAlbum;
            phototmp = photo->copy();
            photolist.append(phototmp);

        }

    }

    KalMimeSource * mime_source = new KalMimeSource(photolist);
    KApplication::clipboard()->setData(mime_source);

}

void KalbumView::editPhotoItem()
{
    PhotoAlbum *photo = new PhotoAlbum;


    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {

            slotPhotoProp(photo->iconitem);

        }

    }
}


void KalbumView::cutPhotoItem()
{
    PhotoAlbum *photo = new PhotoAlbum;
    QList<PhotoAlbum> photolist;

    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {

            PhotoAlbum *phototmp = new PhotoAlbum;
            phototmp = photo->copy();
            photolist.append(phototmp);
            delete photo->iconitem;
            album->remove(photo);
            photo=album->first();

        }
    }

    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {

            PhotoAlbum *phototmp = new PhotoAlbum;
            phototmp = photo->copy();
            photolist.append(phototmp);
            delete photo->iconitem;
            album->remove(photo);

        }
    }

    KalMimeSource * mime_source = new KalMimeSource(photolist);
    KApplication::clipboard()->setData(mime_source);


    KalbumDoc *doc;
    doc=getDocument();
    doc->setModified(true);
    KIconView1->sort();

}


/** Pastes URL information into the App */
void KalbumView::pastePhotoItem()
{
    QString title;
    QMimeSource * mime_source = KApplication::clipboard()->data();

    if (mime_source->provides("application/kalbum"))
    {

        QBuffer buffer(mime_source->encodedData("application/kalbum"));
        buffer.open(IO_ReadOnly);
        QDataStream stream(&buffer);

        int howmany;

        stream>>howmany;

        for (int i=0; i<howmany; i++)
        {
            PhotoAlbum *photo = new PhotoAlbum(stream, 0);

            addPhotoItem(photo);
        }
    }

    KalbumDoc *doc;
    doc=getDocument();
    doc->setModified(true);

}

int KalbumView::countPhoto()
{

    return KIconView1->count();

}


bool KalbumView::savePhotoAlbum(const KURL& url)
{

    QString tmpfile;
    QString checkFileEnd;
    QString stringCount;
    QIconViewItem *positem;
    QDateTime temp_datetime;

    KSimpleConfig *config;
    PhotoAlbum *photo;
    int photo_count=0;

    checkFileEnd = url.fileName();
    checkFileEnd = checkFileEnd.right(7);

    KIO::NetAccess::download( url, tmpfile);
    if (!(checkFileEnd == FILEEX))
    {
        tmpfile+=FILEEX;
    }

    QFile f(tmpfile);

    // if (KIO::NetAccess::exists(url))

    if (f.exists())
    {
        int result = KMessageBox::warningContinueCancel(this, i18n("Do you want to overwrite "
                     "the existing file?"), i18n("Overwrite File"), i18n("OK"), "Overwrite " + tmpfile, false);

        if (result == KMessageBox::Cancel)
        {
            return false;
        }
    }




    if (!f.open(IO_WriteOnly))
    {
        return false;
    }
    f.close();


    config = new KSimpleConfig(tmpfile);
    config->setGroup("Kalbum");
    config->writeEntry("Version", VERSION);

    config->writeEntry("Album Name", credits.albumName);
    config->writeEntry("Web Address", credits.webAddress.url());
    config->writeEntry("Authors Name", credits.authorName);
    config->writeEntry("Authors Email", credits.authorEmail);

    temp_datetime=credits.startDate;
    config->writeEntry("Start Date",	temp_datetime);
    temp_datetime=credits.endDate;
    config->writeEntry("End Date", temp_datetime);

    config->writeEntry("Ignore Dates", (int)credits.ignoredates);

    config->writeEntry("Description", credits.albumDescription);

    for (positem=KIconView1->firstItem(); positem!=0; positem=positem->nextItem())
    {

        photo = getPhoto(positem);
        stringCount.setNum(photo_count++);
        config->setGroup("Photo"+stringCount);
        config->writeEntry("Name", photo->title);
        config->writeEntry("URL", photo->fileurl.url());

        temp_datetime=photo->date;
        config->writeEntry("Date", temp_datetime);

        config->writeEntry("Ignore Date", (int) photo->ignoredate);

        config->writeEntry("Comment", photo->comment);
        config->writeEntry("Selected", photo->selected);

    }

    config->setGroup("Kalbum");
    config->writeEntry("HowMany", photo_count);

    config->sync();
    delete config;

    return true;

}

void KalbumView::colourHTML(const KURL& src, const KURL& des, int saturation, int hue)
{
    QDir d;
    QImage image;
    QImage image_rgb = red_xpm;

    d = src.path();
    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setNameFilter("*.png");
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    while ( (fi = it.current()) != 0 ) {
      image.load(fi->absFilePath());

      image = KImageEffect::modulate(image, image_rgb, false, KImageEffect::Saturation, saturation, KImageEffect::Red);
      image = KImageEffect::modulate(image, image_rgb, false, KImageEffect::HueShift, hue, KImageEffect::Red);

      image.save( des.path() + "/" + fi->fileName(),"PNG", -1);

      ++it;
    }



}




bool KalbumView::exportHTML()
{

    KURL url, images, thumbs, photos, dir, imagepath, htmldes, htmlsrc;
    QString htmldata;

    dir = KGlobal::dirs()->findResourceDir("kalbum_data", "kalbum_html/");
    dir.addPath("kalbum_html");
    dir.addPath("folder");
    imagepath = dir;
    htmlsrc		= dir;

    ExportWeb *exportweb = new ExportWeb;

    if (exportweb->exec()){
    // Export Web Dialoh
      url = exportweb->KURLRequester1->url();
      thumbs 		= url;
      photos 		= url;
      images    = url;
      htmldes		= url;

      thumbs.addPath("thumbs");
      photos.addPath("photos");
      images.addPath("images");

      imagepath.addPath("images");


      if (KIO::NetAccess::mkdir( url)){
        KIO::NetAccess::mkdir( thumbs );
        KIO::NetAccess::mkdir( photos );
//        KIO::NetAccess::mkdir( images );
        KIO::NetAccess::dircopy(imagepath, url);

        if (exportweb->RadioButton1->isChecked())
          colourHTML(imagepath, images, exportweb->saturation->value(), exportweb->hue->value());

        exportThumbs(thumbs);
        exportImages(photos);

        makeHTML(htmlsrc, htmldes);

        return true;

      }
    }

    return false;
}

bool KalbumView::exportHTML(const KURL& url)
{
    KURL thumbs, photos, dir, imagepath, htmldes, htmlsrc;
    QString htmldata;
    thumbs 		= url;
    photos 		= url;
    htmldes		= url;

    thumbs.addPath("thumbs");
    photos.addPath("photos");

    dir = KGlobal::dirs()->findResourceDir("kalbum_data", "kalbum_html/");
    dir.addPath("kalbum_html");
    dir.addPath("folder");
    imagepath = dir;
    htmlsrc		= dir;
    imagepath.addPath("images");



    if (KIO::NetAccess::mkdir( url))
    {
        KIO::NetAccess::mkdir( thumbs );
        KIO::NetAccess::mkdir( photos );
//        KIO::NetAccess::mkdir( imagepath );
        //KIO::NetAccess::dircopy(imagepath, url);

//        exportThumbs(thumbs);
//        exportImages(photos);

//        makeHTML(htmlsrc, htmldes);

        return true;
    }




    return false;

}

bool KalbumView::makeHTML(const KURL& src, const KURL& des)
{
    KURL htmltmp, htmldes;

    QString gen_src = "gen_src.htm";
    QString pho_src = "pho_src.htm";
    QString thm_src = "thm_src.htm";

    PhotoAlbum *photo;
    QIconViewItem *positem;
    QString htmldata;
    QString htmltitle;
    QString htmlphoto;
    QString htmlphoto_src;
    QString temp;
    QString first_photo;
    int no_thm, no_pho, photo_count;
    int i=0;

    QString key_pho = KWDPHCNT;
    QString key_thm = KWDTHCNT;

    htmltmp = src;
    htmltmp.addPath(gen_src);
    htmltitle = readString(htmltmp);

    photo = getPhoto(KIconView1->firstItem());
    first_photo = photo->filename;

    // Read the capabilities of the Web Site from the general page
    if (temp = htmltitle.mid(htmltitle.find(key_pho)+key_pho.length(), MAXPHOTO))
    {
        if (!(no_pho = temp.toInt()))
            return false;                   // false if no photos defined

        temp = htmltitle.mid(htmltitle.find(key_thm)+key_thm.length(), MAXPHOTO);
        if (!(no_thm = temp.toInt()))
            return false;

        photo_count = countPhoto();

        htmltmp = src;
        htmltmp.addPath(thm_src);
        htmldata = readString(htmltmp);

        htmltmp = src;
        htmltmp.addPath(pho_src);
        htmlphoto_src = readString(htmltmp);

        for (positem=KIconView1->firstItem(); positem!=0; positem=positem->nextItem())
        {
            photo 		= getPhoto(positem);
            htmlphoto = htmlphoto_src;

            htmldata 	= makeHTMLthumb(htmldata,	 (i++ % no_thm), photo);

            htmlphoto = makeHTMLphoto(htmlphoto, photo);
            htmlphoto = linkPhoto(htmlphoto, positem);
            htmldes 	= des;
            temp 			= photo->filename;
            temp.append(".htm");
            htmldes.addPath(temp);
            writeString(htmlphoto, htmldes);

            if ((i % no_thm)==0)
            {

                htmldes = des;
                temp =  temp.setNum(i/no_thm);
                temp =  getPrecision(temp);
                temp.prepend(THUMBHTM);
                temp.append(".htm");
                htmldes.addPath(temp);

                htmldata = linkThumb(htmldata, first_photo, (i/no_thm), (photo_count/no_thm) + 1);

                writeString(htmldata, htmldes);

                htmltmp = src;
                htmltmp.addPath(thm_src);
                htmldata = readString(htmltmp);
            }
        }

        if (i % no_thm)
        {
            while (i % no_thm)
                htmldata = makeHTMLthumb(htmldata, (i++ % no_thm));

            htmldes = des;
            temp =  temp.setNum(i/no_thm);
            temp =  getPrecision(temp);
            temp.prepend(THUMBHTM);
            temp.append(".htm");
            htmldes.addPath(temp);

            htmldata = linkThumb(htmldata, first_photo, (i/no_thm), (photo_count/no_thm) + 1);

            writeString(htmldata, htmldes);

        }

        htmldes = des;
        htmldes.addPath("index.htm");
        htmltitle = makeHTMLtitle(htmltitle);
        htmltitle = linkPhoto(htmltitle);
        htmltitle = linkThumb(htmltitle, first_photo);

        writeString(htmltitle, htmldes);

        return true;
    }

    return false;

}

QString KalbumView::linkThumb(QString htmldata, QString first_photo)
{
    QString ins_string;
    int findstt;


    //Change the Quick tabs
    findstt	= htmldata.find("<!-- {Kalbum Link Browse} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;

        htmldata.insert(findstt, first_photo);
    }


    return htmldata;

}




QString KalbumView::linkThumb(QString htmldata, QString first_photo, int page_number, int max_page)
{
    QString ins_string;
    int findstt,findend;

    // Deal with the previous link
    if (page_number == 1)
    {

        findstt	= htmldata.find("<!-- {Kalbum Thumbnail Prev} -->");

        if (findstt>0)
        {
            findend	= htmldata.find("</a>", findstt) + 4;
            htmldata.remove(findstt, findend - findstt);
        }
    }
    else
    {

        findstt	= htmldata.find("<!-- {Kalbum Thumbnail Prev} -->");

        if (findstt>0)
        {
            findstt	= htmldata.find("href=", findstt) + 6;  		// Six including double quotes
            ins_string =  ins_string.setNum(page_number - 1);
            ins_string =  getPrecision(ins_string);
            ins_string.prepend(THUMBHTM);

            htmldata.insert(findstt, ins_string);
        }
    }

    // Deal with the next link
    if (page_number == max_page)
    {

        findstt	= htmldata.find("<!-- {Kalbum Thumbnail Next} -->");

        if (findstt>0)
        {
            findend	= htmldata.find("</a>", findstt) + 4;
            htmldata.remove(findstt, findend - findstt);
        }
    }
    else
    {

        findstt	= htmldata.find("<!-- {Kalbum Thumbnail Next} -->");

        if (findstt>0)
        {
            findstt	= htmldata.find("href=", findstt) + 6;  		// Six including double quotes
            ins_string =  ins_string.setNum(page_number + 1);
            ins_string =  getPrecision(ins_string);
            ins_string.prepend(THUMBHTM);

            htmldata.insert(findstt, ins_string);
        }
    }

    //Change the Quick tabs
    findstt	= htmldata.find("<!-- {Kalbum Link Browse} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;

        htmldata.insert(findstt, first_photo);
    }

    findstt	= htmldata.find("<!-- {Kalbum Link General} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;

        htmldata.insert(findstt, "index");
    }

    return htmldata;

}

QString KalbumView::linkPhoto(QString htmldata, QIconViewItem *positem)
{
    QString ins_string;
    QIconViewItem * temp_positem;
    int findstt,findend;
    PhotoAlbum *photo;


    // Deal with the previous link
    if (!(temp_positem = positem->prevItem()))
    {

        findstt	= htmldata.find("<!-- {Kalbum Photo Prev} -->");

        if (findstt>0)
        {
            findend	= htmldata.find("</a>", findstt) + 4;
            htmldata.remove(findstt, findend - findstt);
        }
    }
    else
    {

        findstt	= htmldata.find("<!-- {Kalbum Photo Prev} -->");

        if (findstt>0)
        {
            findstt	= htmldata.find("href=", findstt) + 6;  		// Six including double quotes
            photo 		 = getPhoto(temp_positem);
            ins_string = photo->filename;
            htmldata.insert(findstt, ins_string);
        }
    }

    // Deal with the next link
    if (!(temp_positem = positem->nextItem()))
    {

        findstt	= htmldata.find("<!-- {Kalbum Photo Next} -->");

        if (findstt>0)
        {
            findend	= htmldata.find("</a>", findstt) + 4;
            htmldata.remove(findstt, findend - findstt);
        }
    }
    else
    {

        findstt	= htmldata.find("<!-- {Kalbum Photo Next} -->");

        if (findstt>0)
        {
            findstt	= htmldata.find("href=", findstt) + 6;  		// Six including double quotes
            photo 		 = getPhoto(temp_positem);
            ins_string = photo->filename;

            htmldata.insert(findstt, ins_string);
        }
    }

    //Change the Quick tabs
    findstt	= htmldata.find("<!-- {Kalbum Link Thumbnails} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;
        ins_string = ins_string.setNum(1);
        ins_string = getPrecision(ins_string);
        ins_string.prepend(THUMBHTM);

        htmldata.insert(findstt, ins_string);
    }

    findstt	= htmldata.find("<!-- {Kalbum Link General} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;

        htmldata.insert(findstt, "index");
    }

    return htmldata;

}

QString KalbumView::linkPhoto(QString htmldata)
{
    QString ins_string;
    int findstt;


    //Change the Quick tabs
    findstt	= htmldata.find("<!-- {Kalbum Link Thumbnails} -->");

    if (findstt>0)
    {
        findstt		 = htmldata.find("<a href=", findstt) + 9;
        ins_string = ins_string.setNum(1);
        ins_string = getPrecision(ins_string);
        ins_string.prepend(THUMBHTM);

        htmldata.insert(findstt, ins_string);
    }

    return htmldata;

}

QString KalbumView::makeHTMLtitle(QString htmldata)
{
    QString ins_string;
    QDateTime generation_time;
    int findstt,findend;

    findstt			= htmldata.find("{Kalbum Document Title}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        htmldata.insert(findstt, credits.albumName);
    }

    findstt			= htmldata.find("{Kalbum Page Title}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        htmldata.insert(findstt, credits.albumName);
    }

    findstt			= htmldata.find("{Kalbum Album Title}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        htmldata.insert(findstt, credits.albumName);
    }

    findstt			= htmldata.find("{Kalbum Album Start Date}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        if (!credits.ignoredates)
          htmldata.insert(findstt, credits.startDate.toString() + " - ");
    }

    findstt			= htmldata.find("{Kalbum Album End Date}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        if (!credits.ignoredates)
          htmldata.insert(findstt, credits.endDate.toString());
    }

    findstt			= htmldata.find("{Kalbum Author Name}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        if (!credits.authorName.isEmpty())
          htmldata.insert(findstt, "Author: " + credits.authorName);
    }

    findstt			= htmldata.find("{Kalbum Author email Link}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        if (!credits.authorEmail.isEmpty())
          htmldata.insert(findstt, "<a HREF=\"mailto:" + credits.authorEmail + "\">");
    }

    findstt			= htmldata.find("{Kalbum Author email}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        if (!credits.authorEmail.isEmpty())
          htmldata.insert(findstt, credits.authorEmail);
    }

    findstt			= htmldata.find("{Kalbum Album Description}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);
        htmldata.insert(findstt, credits.albumDescription);
    }

    findstt			= htmldata.find("{Kalbum Generation Details}");

    if (findstt>0)
    {
        findend			= htmldata.find("}", findstt) + 1;
        htmldata.remove(findstt, findend - findstt);

        ins_string = "Generated by Kalbum v.";
        ins_string.append(VERSION);
        ins_string.append(" on ");
        generation_time = generation_time.currentDateTime();
        ins_string.append(generation_time.toString());

        htmldata.insert(findstt, ins_string);
    }


    return htmldata;

}

QString KalbumView::makeHTMLphoto(QString htmldata, PhotoAlbum *photo)
{
    QString ins_string;
    QString temp;
    int findstt,findend;
    //	int photoszx = PHOTOSZX;
    //	int photoszy = PHOTOSZY;
    QString format = PHOTOFMT;

    // Inserts the general album credits
    htmldata = makeHTMLtitle(htmldata);

    findstt		= htmldata.find("<!-- {Kalbum Photo Image} -->");

    // Inserts Photo Image
    if (findstt>0)
    {
        findstt		= htmldata.find("img src=", findstt) + 9;
        findend		= htmldata.find(">", findstt);

        htmldata.remove(findstt, findend - findstt);

        ins_string = "photos/";
        ins_string.append (photo->filename);
        ins_string.append ("."+format.lower());
        //		temp = temp.setNum(photoszx);
        //		ins_string.append ("\" width=\"");
        //		ins_string.append (temp);
        //		temp = temp.setNum(photoszy);
        //		ins_string.append ("\" height=\"");
        //		ins_string.append (temp);
        ins_string.append ("\"");

        htmldata.insert(findstt, ins_string);
    }

    // Inserts Photo name
    temp 				= "{Kalbum Photo Name}";
    findstt			= htmldata.find(temp);
    findend			= htmldata.find("}", findstt) + 1;
    if (findstt>0)
    {
        htmldata.remove(findstt, findend - findstt);
        if (!photo->title.isEmpty())
          htmldata.insert(findstt, photo->title);
    }

    // Inserts Photo Date
    temp 				= "{Kalbum Photo Date}";
    findstt			= htmldata.find(temp);
    findend			= htmldata.find("}", findstt) + 1;
    if (findstt>0)
    {
        htmldata.remove(findstt, findend - findstt);
        if (!photo->ignoredate)
          htmldata.insert(findstt, photo->date.toString());
    }

    // Inserts Photo Description
    temp 				= "{Kalbum Photo Description}";
    findstt			= htmldata.find(temp);
    findend			= htmldata.find("}", findstt) + 1;
    if (findstt>0)
    {
        htmldata.remove(findstt, findend - findstt);
        if (!photo->comment.isEmpty())
          htmldata.insert(findstt, photo->comment);
    }

    return htmldata;

}


QString KalbumView::makeHTMLthumb(QString htmldata, int photo_number, PhotoAlbum *photo)
{
    QString key_thm_thumb = KWDTHUMB;
    QString key_thm_title = KWDTITLE;
    QString format = THUMBFMT;
    QString ins_string;

    QString temp;
    int findstt,findend;
    int thumbszx = THUMBSZX;
    int thumbszy = THUMBSZY;

    photo_number++;

    temp = temp.setNum(photo_number);
    key_thm_thumb.append(getPrecision(temp));

    // Inserts the general album credits
    htmldata = makeHTMLtitle(htmldata);

    // Find the Kalbum thumbnail entry
    findstt 		= htmldata.find(key_thm_thumb);

    // Edits the link to the photo's HTML
    if (findstt>0)
    {
        findstt			= htmldata.find("<a href=", findstt) + 9;
        ins_string 	= photo->filename;
        ins_string.append (".");
        ins_string.append ("htm\"");

        htmldata.insert(findstt, ins_string);

        // Inserts Photo's thumbnail
        findstt			=	htmldata.find("<img src=", findstt) + 9;
        findend 		= htmldata.find(">", findstt);

        if (findstt>0)
        {
            htmldata.remove(findstt, findend - findstt);

            ins_string = "\"thumbs/";
            ins_string.append (photo->filename);
            ins_string.append ("."+format.lower());
            temp = temp.setNum(thumbszx);
            ins_string.append ("\" width=\"");
            ins_string.append (temp);
            temp = temp.setNum(thumbszy);
            ins_string.append ("\" height=\"");
            ins_string.append (temp);
            ins_string.append ("\" border=\"0\"");

            htmldata.insert(findstt, ins_string);
        }
    }

    // Inserts Photo name
    temp = temp.setNum(photo_number);
    temp = getPrecision(temp);
    temp.prepend("{Kalbum Title=");
    findstt			= htmldata.find(temp);
    findend			= htmldata.find("}", findstt) + 1;
    if (findstt>0)
    {
        htmldata.remove(findstt, findend - findstt);
        temp = temp.setNum(findstt);

        if (!photo->title.isEmpty())
          htmldata.insert(findstt, photo->title);
    }

    return htmldata;

}


QString KalbumView::makeHTMLthumb(QString htmldata, int photo_number)
{
    QString key_thm_thumb = KWDTHUMB;
    QString key_thm_title = KWDTITLE;
    QString format = THUMBFMT;
    QString ins_string;

    photo_number++;

    QString temp;
    int findstt,findend;
    //	int thumbszx = THUMBSZX;
    //	int thumbszy = THUMBSZY;

    temp = temp.setNum(photo_number);
    key_thm_thumb.append(getPrecision(temp));

    // Inserts the general album credits
    htmldata = makeHTMLtitle(htmldata);

    // Find the Kalbum thumbnail entry
    findstt 		= htmldata.find(key_thm_thumb);

    // Removes the link to the photo's HTML
    if (findstt>0)
    {
        findstt			= htmldata.find("<a href=", findstt);
        findend 		= htmldata.find("</a>", findstt) + 4;

        htmldata.remove(findstt, findend - findstt);
    }

    // Removes Photo name
    temp = temp.setNum(photo_number);
    temp = getPrecision(temp);
    temp.prepend("{Kalbum Title=");
    findstt			= htmldata.find(temp);
    findend			= htmldata.find("}", findstt) + 1;
    if (findstt>0)
        htmldata.remove(findstt, findend - findstt);


    return htmldata;

}


QString KalbumView::getPrecision(QString str_number)
{
    int str_length;

    str_length = str_number.length();
    if (str_length < MAXPHOTO)
        for (int i=str_length; i < MAXPHOTO; i++)
            str_number.prepend("0");

    return str_number;

}




bool KalbumView::writeString(QString stringHTML, const KURL& url)
{

    QFile f(url.path());

    if(f.open(IO_WriteOnly))
    {
        QTextStream t(&f);

        t << stringHTML;

    }

    return true;

}

QString KalbumView::readString(const KURL& url)
{
    QString string = "";
    QString tmpfile;

    KIO::NetAccess::download( url, tmpfile);
    QFile f(tmpfile);

    if(f.open(IO_ReadOnly))
    {
        QTextStream t(&f);

        while (!t.eof())
        {
            string = string + t.readLine();
            string.append("\n");
        }
    }

    return string;

}



int KalbumView::exportThumbs(const KURL& url)
{
    QIconViewItem *positem;
    PhotoAlbum *photo;
    int photo_count=0;
    QString format = THUMBFMT;
    QString tmpfile;
    QString thumbnum;
    QImage buffer;
    KPixmap background, thumbnail;
    KURL dir;

    for (positem=KIconView1->firstItem(); positem!=0; positem=positem->nextItem())
    {

        KURL thumbpath=url;
        photo = getPhoto(positem);
        thumbpath.addPath(photo->filename+"."+format.lower());

        KIO::NetAccess::download( photo->fileurl, tmpfile);

        if (buffer.load(tmpfile))
        {
            int thumbSizeX=110;
            int thumbSizeY=90;

            dir = url;
            dir = dir.upURL();
            dir.addPath("images/slide01.png");

            


            background.load(dir.path());

            float s=MAX((float(buffer.size().width())/thumbSizeX),
                    (float(buffer.size().height())/thumbSizeY));

            buffer=buffer.smoothScale(MAX(1,int(buffer.width()/s)),
                                      MAX(1,int(buffer.height()/s)));

            thumbnail.convertFromImage(buffer, QPixmap::Color);

            bitBlt(&background, (background.width()-thumbnail.width())/2,
                    (background.height()-thumbnail.height())/2, &thumbnail, 0, 0,
                    thumbnail.width(), thumbnail.height(), Qt::CopyROP, false);

            background.save(thumbpath.path(), format);

            photo_count++;

        } else {    // else just save the thumbnail(icon) used in the view

          photo->thumbnail.save(thumbpath.path(), format);
          photo_count++;

        }
    }

    return photo_count;
}


int KalbumView::exportImages(const KURL& url)
{

    QIconViewItem *positem;
    PhotoAlbum *photo;
    int photo_count=0;
    QString format = PHOTOFMT;
    QString thumbnum;
    QImage buffer;
    QString tmpfile;

    for (positem=KIconView1->firstItem(); positem!=0; positem=positem->nextItem())
    {

        KURL photopath=url;

        photo = getPhoto(positem);
        photopath.addPath(photo->filename+"."+format.lower());

        KIO::NetAccess::download( photo->fileurl, tmpfile);

        if (buffer.load(tmpfile))
        {
            float s=MAX((float(buffer.size().width())/PHOTOSZX),
                        (float(buffer.size().height())/PHOTOSZY));

            buffer=buffer.smoothScale(MAX(1,int(buffer.width()/s)),
                                      MAX(1,int(buffer.height()/s)));

            buffer.save(photopath.path(), format);
            photo_count++;
        }
    }

    return photo_count;

}



void KalbumView::editMoveUp()
{

    PhotoAlbum *photo, *tempphoto;
    QIconViewItem *newitem, *positem;
    bool lastitem=false;
    QList <PhotoAlbum> photolist;

    if (!allSelected())
    {

        photolist.setAutoDelete(false);

        for (photo=album->last(); photo!=0; photo=album->prev())
        {
            if (photo->iconitem->isSelected())
            {

                photolist.append(photo);

                // Move the insert position along until there's an Unselected icon
                positem=photo->iconitem;
                while (positem->isSelected())
                {
                    if (positem == KIconView1->lastItem())
                    {
                        lastitem =true;
                        positem = KIconView1->firstItem();
                    }
                    else
                    {
                        positem = positem->nextItem();
                    }
                }

                newitem = photo->iconitem;
                photo->iconitem = new QIconViewItem( KIconView1, positem, photo->title, photo->thumbnail);

                if (lastitem)
                {
                    tempphoto = getPhoto(KIconView1->firstItem());
                    delete KIconView1->firstItem();
                    tempphoto->iconitem = new QIconViewItem( KIconView1, KIconView1->firstItem(), tempphoto->title, tempphoto->thumbnail);
                    lastitem = false;
                }

                delete newitem;
                photo=album->last();

            }
        }

        for (photo=photolist.first(); photo !=0; photo=photolist.next())
        {
            photo->iconitem->setSelected(true, true);
        }

        KalbumDoc *doc;
        doc=getDocument();
        doc->setModified(true);

    }
}

void KalbumView::editMoveDown()
{

    PhotoAlbum *photo, *tempphoto;
    QIconViewItem *newitem, *positem;
    bool firstitem=false, seconditem=false;
    QList <PhotoAlbum> photolist;

    if (!allSelected())
    {

        photolist.setAutoDelete(false);

        for (photo=album->first(); photo!=0; photo=album->next())
        {
            if (photo->iconitem->isSelected())
            {

                photolist.append(photo);

                // Move the insert position back until there's an Unselected icon
                positem=photo->iconitem;
                while (positem->isSelected())
                {
                    if (positem == KIconView1->firstItem())
                    {   												// Special case: First Item
                        firstitem = true;
                        positem = KIconView1->lastItem();
                    }
                    else if (positem == KIconView1->firstItem()->nextItem())
                    {     	// Special case: Second Item
                        seconditem = true;
                        positem = positem->prevItem();
                    }
                    else
                    {
                        positem = positem->prevItem();
                    }
                }


                if (firstitem)
                {

                    newitem = photo->iconitem;
                    photo->iconitem = new QIconViewItem( KIconView1, positem->prevItem(), photo->title, photo->thumbnail);

                    tempphoto = getPhoto(KIconView1->lastItem());
                    delete KIconView1->lastItem();
                    tempphoto->iconitem = new QIconViewItem( KIconView1, KIconView1->lastItem()->prevItem(), tempphoto->title, tempphoto->thumbnail);
                    firstitem = false;

                }
                else if (seconditem)
                {

                    newitem = photo->iconitem;
                    photo->iconitem = new QIconViewItem( KIconView1, KIconView1->firstItem(), photo->title, photo->thumbnail);
                    tempphoto = getPhoto(KIconView1->firstItem());
                    delete KIconView1->firstItem();

                    tempphoto->iconitem = new QIconViewItem( KIconView1, KIconView1->firstItem(), tempphoto->title, tempphoto->thumbnail);
                    seconditem=false;

                }
                else
                {

                    newitem = photo->iconitem;
                    photo->iconitem = new QIconViewItem( KIconView1, positem->prevItem(), photo->title, photo->thumbnail);
                }


                delete newitem;
                photo=album->first();

            }
        }

        for (photo=photolist.first(); photo !=0; photo=photolist.next())
        {
            photo->iconitem->setSelected(true, true);
        }

        KalbumDoc *doc;
        doc=getDocument();
        doc->setModified(true);

    }
}

bool KalbumView::loadPhotoAlbum(const KURL& url)
{
    QString tmpfile;
    QString stringCount;
    KSimpleConfig *config;
    QDateTime tmp_date;

    int photo_count=0;

    if (!(url.isValid()))
        return false;

    KIO::NetAccess::download( url, tmpfile);

    QFile f(tmpfile);

    if ((!f.open(IO_ReadOnly)) || (!f.exists()))
    {
        return false;
    }

    f.close();

    config = new KSimpleConfig(tmpfile);
    config->setGroup("Kalbum");

    credits.albumName					= config->readEntry("Album Name");
    credits.webAddress				=	config->readEntry("Web Address");
    credits.authorName				= config->readEntry("Authors Name");
    credits.authorEmail				= config->readEntry("Authors Email");
    credits.albumDescription  =	config->readEntry("Description");
    tmp_date = config->readDateTimeEntry("Start Date");
    credits.startDate					= tmp_date.date();
    tmp_date = config->readDateTimeEntry("End Date");
    credits.endDate						= tmp_date.date();

    credits.ignoredates       = config->readBoolEntry("Ignore Dates");

    photo_count = config->readUnsignedNumEntry("HowMany");

    for (int i=0; i<photo_count; i++)
    {
        PhotoAlbum *photo = new PhotoAlbum;
        stringCount.setNum(i);
        config->setGroup("Photo"+stringCount);
        photo->title = config->readEntry("Name");
        photo->fileurl = config->readEntry("URL");
        tmp_date = config->readDateTimeEntry("Date");
        photo->date =  tmp_date.date();
        photo->ignoredate = config->readBoolEntry("Ignore Date");
        photo->comment = config->readEntry("Comment");
        addPhotoItem(photo);
    }
//    Shows album information whenever a new one is loaded
//    setDocInfo();

    delete config;
    return true;
}


/** Changes properties dialog to represent the next photo */
void KalbumView::nextProp(KalbumItem *item)
{

    QPixmap *buffer = new QPixmap;
    PhotoAlbum *photo;
    QString fname;
    QString tmpfile;

    photo=item->photo;
    photo->comment=item->EditComment->text();
    photo->title=item->LineEditTitle->text();
    photo->iconitem->setText(photo->title);
    photo->date=item->photoDate->getDate();
    photo->iconitem->setSelected(false);
    photo->ignoredate = item->RadioButton1->isChecked();

    photo=nextPhoto(photo);
    photo->iconitem->setSelected(true);

    KIO::NetAccess::download( photo->fileurl, tmpfile);

    if (!buffer->load(tmpfile)){
          buffer = photo->iconitem->pixmap();
    }

    item->PixmapLabel2->setPixmap(*buffer);

    float s=MAX((float(photo->xsize)/ITEMPIXX), (float(photo->ysize)/ITEMPIXY));

    item->PixmapLabel2->setMaximumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );

    item->PixmapLabel2->setMinimumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );


    item->photo = photo;
    item->setCaption(photo->fileurl.fileName());
    item->EditComment->setText(photo->comment);
    item->LineEditTitle->setText(photo->title);
    item->photoDate->setDate(photo->date);
    item->RadioButton1->setChecked(photo->ignoredate);
    //	item->TextLabelSize->setNum(photo->filesize/1024);


}

/** Changes properties dialog to represent the previous photo */
void KalbumView::prevProp(KalbumItem *item)
{

    QPixmap *buffer = new QPixmap;
    PhotoAlbum *photo;
    QString fname;
    QString tmpfile;

    photo=item->photo;
    photo->comment=item->EditComment->text();
    photo->title=item->LineEditTitle->text();
    photo->iconitem->setText(photo->title);
    photo->date=item->photoDate->getDate();
    photo->iconitem->setSelected(false);
    photo->ignoredate = item->RadioButton1->isChecked();

    photo=prevPhoto(photo);
    photo->iconitem->setSelected(true);
    KIO::NetAccess::download( photo->fileurl, tmpfile);

    if (!buffer->load(tmpfile)){
          buffer = photo->iconitem->pixmap();
    }

    item->PixmapLabel2->setPixmap(*buffer);

    float s=MAX((float(photo->xsize)/ITEMPIXX), (float(photo->ysize)/ITEMPIXY));

    item->PixmapLabel2->setMaximumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );

    item->PixmapLabel2->setMinimumSize( QSize (MAX(1,int(photo->xsize/s)),
                                        MAX(1,int(photo->ysize/s))) );

    item->photo = photo;
    item->setCaption(photo->fileurl.fileName());
    item->EditComment->setText(photo->comment);
    item->LineEditTitle->setText(photo->title);
    item->photoDate->setDate(photo->date);
    item->RadioButton1->setChecked(photo->ignoredate);
    //	item->TextLabelSize->setNum(photo->filesize/1024);

}


/** Standard dnd from the QT Docs */
void KalbumView::dragEnterEvent(QDragEnterEvent *event)
{
    event->accept(
        QTextDrag::canDecode(event) ||
        QImageDrag::canDecode(event)
    );

}


bool KalbumView::allSelected()
{
    PhotoAlbum *photo;
    for (photo=album->first(); photo!=0; photo=album->next())
        if (!(photo->iconitem->isSelected()))
            return false;

    return true;

}

void KalbumView::slotRHClick()
{

    popIconView->exec(QCursor::pos());

}

void KalbumView::slotCut()
{

    cutPhotoItem();

}

void KalbumView::slotCopy()
{

    copyPhotoItem();

}

void KalbumView::slotPaste()
{

    pastePhotoItem();

}

void KalbumView::slotMoveLeft()
{

    editMoveDown();

}

void KalbumView::slotMoveRight()
{

    editMoveUp();

}

void KalbumView::slotEditPhoto()
{

    editPhotoItem();

}



void KalbumView::slotDropEvent( QDropEvent *event, const QValueList<QIconDragItem>& )
{

    QString title;
    int found=0;

    //	QMimeSource * mime_source = KApplication::clipboard()->data();
    KURL::List::ConstIterator it;
    KURL::List url;


    //		if (QIconDrag::canDecode(event)) {
    //		KMessageBox::information(this, "Dnd Info", "Dropped a KIcon.");
    //		}

    /*  if (event->action() == QDropEvent::Copy)
        KMessageBox::information(this, "Dnd Copy", "DnD Copy!");
      else
        KMessageBox::information(this, "Dnd Move", "DnD Move!");
    */
    event->acceptAction();


    if (event->provides("application/kalbum"))
    {
        QPoint drop_pos;
        int src_x, src_y;

        drop_pos = event->pos();
        src_x    = drop_pos.x();
        src_y    = drop_pos.y();

        PhotoAlbum *photo_des;

        if ((KIconView1->count()) == 0)
        {                            // If there are no Icons in the View

            found  = 1;

            QBuffer buffer(event->encodedData("application/kalbum"));
            buffer.open(IO_ReadOnly);
            QDataStream stream(&buffer);

            int howmany;

            stream>>howmany;

            for (int i=0; i<howmany; i++)
            {
                PhotoAlbum *photo = new PhotoAlbum(stream, 0);
                addPhotoItem(photo, album->first());
            }

            if (event->source())
            {
                PhotoAlbum *photo = new PhotoAlbum;
                for (photo=album->first(); photo!=0; photo=album->next())
                {
                    if (photo->iconitem->isSelected())
                    {
                        deletePhotoItem(photo);
                        photo=album->first();
                    }
                }
            }
        }

        for (photo_des=album->last(); found==0 && photo_des!=0; photo_des=album->prev())
        {

            //    The next (if) calculates where the drop should be inserted into the List of photos.
            //    If the drop's y coord is greater than the current icon, it's the correct row.
            //    If the drop's x coord is greater than the current icon, is's the correct column.

            if ((src_y > (photo_des->iconitem->y())) && ( src_x > (photo_des->iconitem->x())))
            {

                PhotoAlbum *photo_tmp;
                photo_tmp = photo_des;
                found  = 1;

                QBuffer buffer(event->encodedData("application/kalbum"));
                buffer.open(IO_ReadOnly);
                QDataStream stream(&buffer);

                int howmany;

                stream>>howmany;

                for (int i=0; i<howmany; i++)
                {                                // Makes sure Icons are insterted in the right order
                    if ((photo_des->iconitem) == (KIconView1->lastItem()))
                    {
                        PhotoAlbum *photo = new PhotoAlbum(stream, 0);
                        addPhotoItem(photo);
                        photo_des = getPhoto(KIconView1->lastItem());
                    }
                    else if (i==0)
                    {
                        PhotoAlbum *photo = new PhotoAlbum(stream, 0);
                        photo_tmp = getPhoto(photo_tmp->iconitem->nextItem());
                        addPhotoItem(photo, photo_tmp);
                    }
                    else
                    {
                        PhotoAlbum *photo = new PhotoAlbum(stream, 0);
                        addPhotoItem(photo, photo_tmp);
                    }
                }

                if (event->source())                              // Duplicated delete loops to catch every selection up/down
                {
                    PhotoAlbum *photo = new PhotoAlbum;
                    for (photo=album->last(); photo!=0; photo=album->prev())
                    {
                        if (photo->iconitem->isSelected())
                        {
                            deletePhotoItem(photo);
                            photo=album->last();
                        }
                    }
                }

                if (event->source())
                {
                    PhotoAlbum *photo = new PhotoAlbum;
                    for (photo=album->first(); photo!=0; photo=album->next())
                    {
                        if (photo->iconitem->isSelected())
                        {
                            deletePhotoItem(photo);
                            photo=album->first();
                        }
                    }
                }

                //  Need to take into account, the space before rows, and the space before the top left icon
            }

            else if ((src_y < ((KIconView1->firstItem()->y())+THUMBSZY)) && (src_x < (KIconView1->firstItem()->x())))
            {   // Before the top left (first icon)

                PhotoAlbum *photo_tmp;
                photo_tmp = getPhoto(KIconView1->firstItem());
                found  = 1;

                QBuffer buffer(event->encodedData("application/kalbum"));
                buffer.open(IO_ReadOnly);
                QDataStream stream(&buffer);

                int howmany;

                stream>>howmany;


                for (int i=0; i<howmany; i++)
                {                                // Makes sure Icons are insterted in the right order

                    PhotoAlbum *photo = new PhotoAlbum(stream, 0);
                    addPhotoItem(photo, photo_tmp);

                }

                if (event->source())                                 // Duplicated delete loops to catch every selection up/down
                {
                    PhotoAlbum *photo = new PhotoAlbum;
                    for (photo=album->first(); photo!=0; photo=album->next())
                    {
                        if (photo->iconitem->isSelected())
                        {
                            deletePhotoItem(photo);
                            photo=album->first();
                        }
                    }
                }

                if (event->source())
                {
                    PhotoAlbum *photo = new PhotoAlbum;
                    for (photo=album->last(); photo!=0; photo=album->prev())
                    {
                        if (photo->iconitem->isSelected())
                        {
                            deletePhotoItem(photo);
                            photo=album->last();
                        }
                    }
                }
            }
        }
    }
    else if (KURLDrag::decode(event, url))
    {
        for (it = url.begin(); it != url.end(); ++it)
        {
            if((*it).isValid())
            {
                addPhotoItem(*it);
            }
        }
    }

    KalbumDoc *doc;
    doc=getDocument();
    doc->setModified(true);

}


void KalbumView::slotSelection()
{

    PhotoAlbum *photo;
    int i=0, total=0;
    KalbumApp *theApp=(KalbumApp *) parentWidget();

    for (photo=album->first(); photo!=0; photo=album->next())
    {
        if (photo->iconitem->isSelected())
        {

            i++;
            total++;

        }
        else
        {

            total++;
        }
    }

    if (i)
    {
        theApp->editCopy->setEnabled(true);
        theApp->editCut->setEnabled(true);
        theApp->editMoveUp->setEnabled(true);
        theApp->editMoveDown->setEnabled(true);
        theApp->editPhoto->setEnabled(true);
        popIconView->setItemEnabled(POPUPCPY, true);
        popIconView->setItemEnabled(POPUPCUT, true);

        popIconView->setItemEnabled(POPUPLFT, true);
        popIconView->setItemEnabled(POPUPRGT, true);
        popIconView->setItemEnabled(POPUPEDT, true);

        if (i==total)
        {
            theApp->editMoveUp->setEnabled(false);
            theApp->editMoveDown->setEnabled(false);

            popIconView->setItemEnabled(POPUPLFT, false);
            popIconView->setItemEnabled(POPUPRGT, false);
        }

        if (i>1)
        {
            theApp->editPhoto->setEnabled(false);
        }

    }
    else
    {
        theApp->editCopy->setEnabled(false);
        theApp->editCut->setEnabled(false);
        theApp->editMoveUp->setEnabled(false);
        theApp->editMoveDown->setEnabled(false);
        theApp->editPhoto->setEnabled(false);

        popIconView->setItemEnabled(POPUPCPY, false);
        popIconView->setItemEnabled(POPUPCUT, false);

        popIconView->setItemEnabled(POPUPLFT, false);
        popIconView->setItemEnabled(POPUPRGT, false);
        popIconView->setItemEnabled(POPUPEDT, false);
    }


}





