/***************************************************************************
                          kalbumview.h  -  description
                             -------------------
    begin                : Tue Jan 22 14:13:58 CET 2002
    copyright            : (C) 2002 by Graham Morrison
    email                : kalbum@paldandy.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KALBUMVIEW_H
#define KALBUMVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qwidget.h>
#include <qlist.h>

//include files added by GMM
#include <kpixmap.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kstddirs.h>
#include <kpopupmenu.h>
#include "kalbumdoc.h"
#include "photoalbum.h"
#include "kalbumitem.h"
#include "kalbumcredits.h"
#include "kalbumcreditsview.h"
#include "kalbumviewdlg.h"
#include "kalmimesource.h"


class KalbumDoc;

/** The KalbumView class provides the view widget for the KalbumApp instance.
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KalbumView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KalbumApp class to manipulate and display
 * the document structure provided by the KalbumDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KalbumView : public KalbumViewDlg
{
    Q_OBJECT
public:
    /** Constructor for the main view */
    KalbumView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KalbumView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KalbumApp instance as a parent
     * widget to get to the window document pointer by calling the KalbumApp::getDocument() method.
     *
     * @see KalbumApp#getDocument
     */
    KalbumDoc *getDocument() const;

    /** Returns photo data for selected Icon */
    PhotoAlbum *getPhoto(QIconViewItem *chosen);

    PhotoAlbum *nextPhoto(PhotoAlbum *photo);
    PhotoAlbum *prevPhoto(PhotoAlbum *photo);

    /** DnD functionality */
    void dragEnterEvent(QDragEnterEvent *event);

    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);

    bool addPhotoItem(const KURL& url=0);
    bool addPhotoItem(PhotoAlbum *photo);
    bool addPhotoItem(PhotoAlbum *photo, PhotoAlbum *before);
    bool allSelected();
    void setSelected();
    void copyPhotoItem();
    void cutPhotoItem();
    void pastePhotoItem();
    void editPhotoItem();
    void deletePhotoItem(PhotoAlbum *photo);
    void editMoveUp();
    void editMoveDown();
    void deleteAll();
    void setDocInfo();
    bool savePhotoAlbum(const KURL& url);
    bool loadPhotoAlbum(const KURL& url);
    void colourHTML(const KURL& src, const KURL& des, int saturation, int hue);
    bool exportHTML(const KURL& url);
    bool exportHTML();
    bool makeHTML(const KURL& src, const KURL& des);
    int	 exportThumbs(const KURL& url);
    int	 exportImages(const KURL& url);
    int  countPhoto();
    void nextProp(KalbumItem *item);
    void prevProp(KalbumItem *item);
    bool writeString(QString stringHTML, const KURL& url);
    QString linkPhoto(QString htmldata);
    QString linkPhoto(QString htmldata, QIconViewItem *positem);
    QString linkThumb(QString htmldata, QString first_photo);
    QString linkThumb(QString htmldata, QString first_photo, int photo_number, int max_photo);
    QString readString(const KURL& url);
    QString getPrecision(QString str_number);
    QString makeHTMLthumb(QString htmldata, int photo_number);
    QString makeHTMLthumb(QString htmldata, int photo_number, PhotoAlbum *photo);
    QString makeHTMLphoto(QString htmldata, PhotoAlbum *photo);
    QString makeHTMLtitle(QString htmldata);
    QList<PhotoAlbum> getPhotoData();
public slots:
    /** Enables the user to get a better look at the photo and to add their own comments.
    */
    void slotPhotoProp(QIconViewItem *chosen);
    void slotDropEvent (QDropEvent *event, const QValueList<QIconDragItem>& list);
    void slotSelection();
    void slotRHClick();
    void slotCut();
    void slotCopy();
    void slotPaste();
    void slotMoveLeft();
    void slotMoveRight();
    void slotEditPhoto();

public:
    /** the PhotoAlbum data
    */
    static QList<PhotoAlbum> *album;
    static KalbumCredits credits;

protected:

    KPopupMenu* popIconView;


};

#endif // KALBUMVIEW_H
